# coding: utf-8

import json

from com.jayway.jsonpath import JsonPath
from fias._fias_orm import addrobjCursor, houseCursor


try:
    from ru.curs.showcase.util import XMLJSONConverter
except:
    from ru.curs.celesta.showcase.utils import XMLJSONConverter

try:
    from ru.curs.showcase.core.jython import JythonDTO
except:
    from ru.curs.celesta.showcase import JythonDTO


def addressSearchForm(context, main, add, filterinfo=None, session=None, elementId=None):

    xformsdata = {
        "schema": {
                "@xmlns": "",
                "address": addressSearchDict().get('address')
            }
        }

    xformssettings = {"properties": {"event": []}}

    jsonData = XMLJSONConverter.jsonToXml(json.dumps(xformsdata))
    jsonSettings = XMLJSONConverter.jsonToXml(json.dumps(xformssettings))
    return JythonDTO(jsonData, jsonSettings)


def addressSearchDict():
    addressSearchDictionary = {
        "address": {
            "info": {
                "@searchString": "",
                "@address": "",
                "@postcode": "",
                "@additional": "",
                "@houseNotFound": "false",
                "@addressNotFound": "false"
            },
            "fias": {
                "region": {
                    "@aoid": "",
                    "@id": "",
                    "@name": "",
                    "@nodeName": "region",
                    "@level": "1",
                    "@parentId": "",
                    "@postcode": "",
                    "@regioncode": ""
                },
                "district": {
                    "@aoid": "",
                    "@id": "",
                    "@name": "",
                    "@nodeName": "district",
                    "@level": "2",
                    "@parentId": "",
                    "@postcode": ""
                },
                "area": {
                    "@aoid": "",
                    "@id": "",
                    "@name": "",
                    "@nodeName": "area",
                    "@level": "3",
                    "@parentId": "",
                    "@postcode": ""
                },
                "city": {
                    "@aoid": "",
                    "@id": "",
                    "@name": "",
                    "@nodeName": "city",
                    "@level": "4",
                    "@parentId": "",
                    "@postcode": ""
                },
                "cityArea": {
                    "@aoid": "",
                    "@id": "",
                    "@name": "",
                    "@nodeName": "cityArea",
                    "@level": "5",
                    "@parentId": "",
                    "@postcode": ""
                },
                "locality": {
                    "@aoid": "",
                    "@id": "",
                    "@name": "",
                    "@nodeName": "locality",
                    "@level": "6",
                    "@parentId": "",
                    "@postcode": ""
                },
                "street": {
                    "@aoid": "",
                    "@id": "",
                    "@name": "",
                    "@nodeName": "street",
                    "@level": "7",
                    "@parentId": "",
                    "@postcode": ""
                },
                "house": {
                    "@id": "",
                    "@name": "",
                    "@level": "99",
                    "@nodeName": "house",
                    "@parentId": "",
                    "@postcode": ""
                }
            }
        }
    }

    return addressSearchDictionary


def addressEditDict(context, table):
    '''загружает адрес из таблицы table в instance карточки для редактирования'''
    aoDictionary = addressSearchDict()
    aoDictionary["address"]["info"]["@address"] = table.address
    aoDictionary["address"]["info"]["@postcode"] = table.postcode
    aoDictionary["address"]["info"]["@additional"] = table.address_additional
    aoDictionary["address"]["info"]["@houseNotFound"] = unicode(table.house_id is None).lower()
    aoDictionary["address"]["info"]["@addressNotFound"] = unicode(table.street_id is None).lower()

    addrobj = addrobjCursor(context)

    if addrobj.tryGet(table.aoid):
        level = addrobj.aolevel
        aolist = aoDictionary["address"]["fias"].items()
        # отсортировали список начиная с улицы
        sortedFiasLevels = sorted(aolist, key=lambda k: k[1]['@level'], reverse=True)
        for nodeName, fiasLevel in sortedFiasLevels:
            # исключая дом
            if fiasLevel['@level'] == unicode(level):
                aoDictionary["address"]["fias"][nodeName]["@aoid"] = addrobj.aoid
                aoDictionary["address"]["fias"][nodeName]["@id"] = addrobj.aoguid
                aoDictionary["address"]["fias"][nodeName]["@parentId"] = addrobj.parentguid
                aoDictionary["address"]["fias"][nodeName]["@name"] = u"%s %s" % (addrobj.formalname, addrobj.shortname)
                if fiasLevel["@nodeName"] == "region":
                    aoDictionary["address"]["fias"][nodeName]["@regioncode"] = addrobj.regioncode
                addrobj.setComplexFilter(u'''aoguid='%s' and actstatus=1''' % addrobj.parentguid)
                if addrobj.tryFirst():
                    level = addrobj.aolevel
                else:
                    break

        if table.house_id:
            house = houseCursor(context)
            house.get(table.house_id)
            housenum = house.housenum  # номер дома(домовладения, участка)
            if house.eststatus == 2:
                houseType = u'д '  # дом
            elif house.eststatus == 3:
                houseType = u'двлд '  # домовладение
            elif house.eststatus == 1:
                houseType = u'влд '  # владение
            elif house.eststatus == 4:
                houseType = u'уч '  # участок
            else:
                houseType = u''

            buildnum = 1 if house.buildnum else 0  # номер корпуса
            strucnum = 1 if house.strucnum else 0  # номер строения

            aoDictionary["address"]["fias"]["house"]["@name"] = houseType + unicode(housenum) + buildnum * (u" корп." + unicode(house.buildnum)) + strucnum * (u" стр. " + unicode(house.strucnum))
            aoDictionary["address"]["fias"]["house"]["@id"] = house.houseid
            aoDictionary["address"]["fias"]["house"]["@parentId"] = house.aoguid  # гуид адреса
            aoDictionary["address"]["fias"]["house"]["@postcode"] = house.postalcode

    return aoDictionary


def saveAdress(table, addrData):
    '''сохраняет адрес в 5 полей в таблице'''
    info = addrData["info"]
    table.address = info["@address"]
    table.postcode = info["@postcode"]
    if info["@addressNotFound"] == 'true':
        table.aoid = None
        table.house_id = None
        table.region_id = None
        table.region_name = None
        table.city_id = None
        table.city_name = None
        table.street_id = None
        table.street_name = None
        table.address_additional = None

    elif info["@addressNotFound"] == 'false':
        # самый большой уровень из заполненного по адресному объекту
        aolist = JsonPath.read(addrData, "$.fias.*")
        # отсортировали список начиная с улицы
        sortedFiasLevels = sorted(aolist, key=lambda k: k['@level'], reverse=True)
        ao = None
        street_dict = dict()
        city_dict = dict()
        region_dict = dict()

        for fiasLevel in sortedFiasLevels:

            if not fiasLevel["@id"]:
                continue
            # исключая дом
            if fiasLevel['@nodeName'] != 'house' and not ao:
                ao = fiasLevel["@aoid"]

            if fiasLevel['@nodeName'] == 'street':
                street_dict = {"id": fiasLevel["@aoid"],
                               "name": fiasLevel["@name"]}

            elif fiasLevel["@nodeName"] in ("locality", "city"):
                city_dict = {"id": fiasLevel["@aoid"],
                             "name": fiasLevel["@name"]}

            elif fiasLevel["@nodeName"] == "region":
                region_dict = {"id": fiasLevel["@aoid"],
                               "name": fiasLevel["@name"]}

        table.aoid = ao
        table.address_additional = info["@additional"]
        table.region_id = region_dict.get("id")
        table.region_name = region_dict.get("name")
        table.city_id = city_dict.get("id")
        table.city_name = city_dict.get("name")
        table.street_id = street_dict.get("id")
        table.street_name = street_dict.get("name")

        table.house_id = addrData["fias"]["house"]["@id"] or None
    else:
        raise Exception(u'что-то пошло не так')
